//модуль для клиента
'use strict';
// это клиент

// Зависимости
let app = require('express')();
let timestamp = require('time-stamp');
const logSymbols = require('log-symbols');
let _ = require('lodash');
let ip = require('ip');
let nanoid = require('nanoid');
let axios = require('axios');

// Конфигурация
const wsName = require('../../package.json').name;
const wsVersion = require('../../package.json').version;
const wsClientConfig = require('../../wsClientConfig.json');
const wsCoreKey = wsClientConfig.wsCoreKey;
const wsPort = wsClientConfig.wsPort;
const wsEventBusAddress = wsClientConfig.wsEventBusAddress;

const instanceID = nanoid(10);

// Характеристики инстанса в сессии сокет-соединения
let socket = require('socket.io-client')(wsEventBusAddress,{
    query: {
        token: wsCoreKey, //jwt
        wsName: wsName,
        instanceID: instanceID
    }
});

// Характеристики инстанса отправляемые сервисам ядра
let wsStartData = {
    wsName: wsName,
    instanceIP: ip.address(),
    instancePort: wsPort,
    instanceID: instanceID,
    instanceVer: wsVersion
};

// Отправляем данные о запущенном сервисе
socket.on('connect', function(data) {
    socket.emit('wsStart', wsStartData);
});

let msSocket = {
    on: function (eventName, onEventAction) {
        let personalEventName = `${wsStartData.instanceID}/${eventName}`;
        //console.log(personalEventName);
        function step1 (data){
            //console.log(`Получено сообщение с идентификатором: ${data.eventId}`);
            let dataToChangeEventState = {'eventId':data.eventId,'eventState':'received'};
            socket.emit('changeEventState', dataToChangeEventState);
        }
        async function step2 (data){
            try {
                await onEventAction(data);
                step3(data);
            } catch (err) {
                //console.log(`Ошибка обработки по сообщению с идентификатором: ${data.eventId}`);
                let dataToChangeEventState = {'eventId':data.eventId,'eventState':'error'};
                socket.emit('changeEventState', dataToChangeEventState);
            }
        }
        function step3 (data){
            //console.log(`Выполнена обработка по сообщению с идентификатором: ${data.eventId}`);
            let dataToChangeEventState = {'eventId':data.eventId,'eventState':'succsess'};
            socket.emit('changeEventState', dataToChangeEventState);
        }
        function run (data){
            step1(data);
            step2(data);
        }
        socket.on(personalEventName, function(data){
            run(data)
        });
        // Регистрация подписки в MQ всех обработчиков, принадлежащих сервису
        socket.emit('regMeOnMQ', {wsName:wsStartData.wsName, instanceID:wsStartData.instanceID, eventName:eventName, personalEventName: personalEventName});
    }
};

async function callAxiGet(axiosPath) {
    const axiosRes = await axios.get(axiosPath);
    return axiosRes.data
}

app.listen(wsPort, function(){
    console.log(`${timestamp('[DD-MM-YYYY HH:mm:ss.ms]')} ${logSymbols.success} Сервис ${wsName} запущен на ${wsPort} порту`)
});

module.exports = {
    callAxiGet: async function(axiosPath) {
    const axiosRes = await axios.get(axiosPath);
    return axiosRes.data
    },
    msSocket: msSocket
};
